-- Craft Mats plugin by David Down
-- coding: utf-8 'ä
import "Turbine"
import "Turbine.Gameplay"
import "Vinny.Common"
import "Vinny.Common.EII_ID"
import "Vinny.CraftMats.CM_Data"
import "Vinny.CraftMats.CM_Window"

function print(text) Turbine.Shell.WriteLine("<rgb=#00FFFF>CM:</rgb> "..text) end
function printe(text) print("<rgb=#FF6040>Error: "..text.."</rgb>") end
function printh(text) print("<rgb=#00FF00>"..text.."</rgb>") end

import "Vinny.Common.Help"

xlink = "<Examine:IIDDID:0x0000000000000000:0x700%s>[%s]<\\Examine>"
xpat = "<Examine:IIDDID:0x0%x+:0x700(%x+)>%[(.-)%]<\\Examine>"
local tpat = "^(%a)(%d%d?)$"
local npat = "^n(%d%d?)$"

Plugins.CraftMats.Open = function(sender,args)
	CM_window:SetVisible( true )
	CM_window:SetZOrder( 2 )
end

RN = {}
for id,t in pairs(Mats) do
	RN[t.N] = id
end

function CM_Find(name)
	name = name:lower()
	for id,item in pairs(Mats) do
		if name==item.N:lower() then return id,item end
	end
end

function CM_Inv(vault,vname)
  local sname = vname or pname
  -- remove old player inventory
  for id,locs in pairs(Inv) do
    locs[sname] = nil
  end
  -- add current player inventory
  local pack = vault or player:GetBackpack()
  local cnt = vault and vault:GetCapacity() or pack:GetSize()
  local nr = 0
  for i=0,cnt do
    local item = pack:GetItem(i)
    if item then
      local info = item:GetItemInfo()
      local name = info:GetName()
      local id = CM_Find(name)
      if id then
        local locs = Inv[id]
        if not locs then
          locs = {}
          Inv[id] = locs
        end
        local cnt = locs[sname] or 0
        locs[sname] = cnt + item:GetQuantity()
        nr = nr + 1
  --      print("Found "..item:GetQuantity().." "..name)
      end
    end
  end
  print("Found "..nr.." stacks of crafting items.")
end

CM_Settings = Turbine.PluginData.Load(Turbine.DataScope.Server,"CraftMats_Settings")
local CMv = "Craft Mats "..Plugins["CraftMats"]:GetVersion()
if type(CM_Settings) ~= "table" then
	CM_Settings = {names=true, player={}, inv={} }
    print(CMv..", settings initialized.")
else print(CMv..", settings loaded.") end
if CM_Settings.auto then
	CM_window:SetPosition(CM_Settings.auto.x, CM_Settings.auto.y)
	CM_window:SetVisible( true )
end

if CM_Settings.names==nil then CM_Settings.names=true end -- default to true
if CM_Settings.inv==nil then CM_Settings.inv={} end
if CM_Settings.prof==nil then -- Add professions?
  CM_Settings.prof = {}
  for i,n in pairs(Prof) do
    if n~="Unknown" and n~="Undefined" then CM_Settings.prof[n] = {} end
  end
end
Inv = CM_Settings.inv
player = Turbine.Gameplay.LocalPlayer.GetInstance()
pname = player:GetName()
if pname:sub(1,1)=="~" then
	printe("Session Play detected.")
	return
end
plvl = player:GetLevel()
pAttr = player:GetAttributes()
local found = CM_Settings.player[pname]
if type(found) ~= "table" then
  CM_Settings.player[pname] = {}
	Turbine.PluginData.Save(Turbine.DataScope.Server,"CraftMats_Settings",CM_Settings)
end
CM_Settings.player[pname].lvl = plvl
if not found then print("Added player name, "..pname); CM_Inv() end

function CM_List(str,vault,vname)
  local cat,tier = str:match(tpat)
  local prof = cat and Cat[cat]
  if not prof then printe("Invalid category.") return end
  tier = tonumber(tier)
  if tier>#Tier then printe("Invalid tier.") return end
  local t = tier>0 and ' '..Tier[tier] or ''
  printh(prof..t.." mats in "..vname..':')
  local pt,fnd,ppt
  if cat~='a' then
    if tier==0 then
      ppt = CM_Settings.prof[prof]
    end
  else cat = '' end
  local pack = vault or player:GetBackpack()
  local cnt = vault and vault:GetCapacity() or pack:GetSize()
  for i=0,cnt do
    local item = pack:GetItem(i)
    if item then
      local info = item:GetItemInfo()
      local name = info:GetName()
      local id,mat = CM_Find(name)
      if id then
        local p,l,T
        if cat~='' then
          p,_,l = mat.C:find(cat.."(%d?)")
          T = l~='' and tonumber(l) or mat.T
        else T = mat.T end
        if (tier==0 or tier==T) and (p or cat=='') and
              (mat.P and mat.P>=0 or CM_Settings.rare) then
          local s = tier==0 and cat or ''
          if cat=='' then 
            s = mat.C
            ppt = CM_Settings.prof[Cat[s:sub(1,1)]]
           end
          if tier==0 then s = s..T end
          if ppt then
            local ps = ''
            for pn,pt in pairs(ppt) do
              if T>pt.ml and T<=pt.pl+1 then
                local c= (pt.pl+1==T or pt.ml+1==T) and '00' or 'FF'
                if ps~='' then ps = ps..',' end
                ps = ps.."<rgb=#"..c.."FF00>"..pn.."</rgb>"
              end
            end
            if ps~='' then s = s..' ('..ps..')' end
          end
          fnd = true
          print('  '..item:GetQuantity()..' '..string.format(xlink,id,mat.N)..' '..s)
        end
      end
    end
  end
  if not fnd then print("(none found)") end
end

function CM_Print(id,item)
  local link = string.format(xlink,id,item.N)
  if item.P then
    print(link.." is a tier "..item.T.." ("..Tier[item.T]..") "..Proc[item.P].." material.")
  else print(link.." is a tier "..item.T.." Guild rep item.") end
end

function ab(a,b) -- Sort players by profession
  local at = pt[a]
  local bt = pt[b]
  if at.pl>bt.pl then return true
  elseif at.pl<bt.pl then return false
  else return at.ml>bt.ml end
end

function CM_Prof(n)
  if Cat[n] then n = Cat[n] end
  pt = CM_Settings.prof[n]
  if not pt then printe("No such profession, "..n) return end
  printh(n.." tiers in progress:")
  local l = {}
  for pn in pairs(pt) do
    table.insert(l,pn)
  end
  table.sort(l, ab)
  for i,pn in ipairs(l) do
    print("  "..pn..": "..(pt[pn].pl+1)..'-'..(pt[pn].ml+1))
  end
end

function ProfLvl(id)
  local ProfInfo = pAttr:GetProfessionInfo(id)
  if not ProfInfo then return 
  else return ProfInfo:GetProficiencyLevel(),ProfInfo:GetMasteryLevel() end
end

-- Update mastery levels
function cVoc(pv)
  if(pv) then print("Checking current professions") end
  local pcnt = 0
  for p,n in pairs(Prof) do
    local pl,ml = ProfLvl(p)
    if pl then
      pcnt = pcnt+1
      local pp = {}
      pp.pl = pl
      pp.ml = ml
      CM_Settings.prof[n][pname] = pp
      if(pv) then
        local txt = n..' = '..(ml+1)
        if pl>ml and pl<#Tier then txt = txt..'-'..(pl+1) end
        print(txt)
      end
    else CM_Settings.prof[n][pname] = nil end
  end
  if pcnt==0 then print("No professions found!") end
end

cVoc() -- Make sure the profession levels are current

function CM_Uses(id,item)
  CM_Print(id,item)
  local locs = Inv[id]
  if locs then
    for name,cnt in pairs(locs) do
      print(name.." has "..cnt)
    end
  end
  for p,tu in item.C:gmatch("(%a)(%d?)") do
    local prof,t,s = Cat[p],(tu and tonumber(tu)) or item.T,''
    for pn,pt in pairs(CM_Settings.prof[prof]) do
      if t<=pt.pl+1 then
        local pns = pn
        if t==pt.ml+1 or t==pt.pl+1 then pns = "<rgb=#00FF00>"..pns.."</rgb>" 
        elseif t>pt.ml and t<=pt.pl then pns = "<rgb=#FFFF00>"..pns.."</rgb>" end
        s = s~='' and s..', '..pns or pns
      end
    end
    print(prof.." uses: "..(s~='' and s or "(None)"))
  end
end

function CM_xp(v,mat)
  local p,l,T
  for c,t in pairs(v) do
    p,_,l = mat.C:find(c.."(%d?)")
    T = l~='' and l or mat.T
    if p and (T==t.pl or T==t.ml) then return c end
  end
end

CM_Command = Turbine.ShellCommand()

function CM_Command:Execute( cmd,args)
	if cmd=="cm" and (args=="help" or args=="?") then
		Vinny.Common.Help(help,args)
		return
	end
	if cmd=="cm?" then
		Vinny.Common.Help(help,"??")
		return
	end
	if cmd=="cmi" then
    if args=="?" then
      Vinny.Common.Help(help,cmd)
      return
    end
    if args== '' then
      print("Adding backpack items:")
      CM_Inv()
    else CM_List(args,nil,"inventory") end
    return
	end
	if cmd=="cmv" then
    if args=="?" then
      Vinny.Common.Help(help,cmd)
      return
    end
		local vault = player:GetVault()
		if vault:IsAvailable() then
      if args== '' then
        print("Adding Vault items:")
        CM_Inv(vault,pname.."(v)")
      else CM_List(args,vault,"vault") end
		else printe("Vault not available.") end
		return
	end
	if cmd=="cms" then
    if args=="?" then
      Vinny.Common.Help(help,cmd)
      return
    end
		local shared = player:GetSharedStorage()
		if shared:IsAvailable() then
      if args== '' then
        print("Adding Shared Storage items:")
        CM_Inv(shared,"(Shared)")
      elseif args== 'xp' then
        local v = {}
        for p,n in pairs(Prof) do
          local pl,ml = ProfLvl(p)
          if pl then v[cat[n]] = {pl=pl+1, ml=ml+1} end
        end
        printh("Craft XP mats in shared storage:")
        local fnd
        for i=0,shared:GetCapacity() do
          local item = shared:GetItem(i)
          if item then
            local info = item:GetItemInfo()
            local id,mat = CM_Find(info:GetName())
            if id and (mat.P and mat.P>=0 or CM_Settings.rare) then
              c = CM_xp(v,mat)
              if c then
                local lnk = string.format(xlink,id,mat.N)
                print('  '..item:GetQuantity()..' '..lnk..' '..c..mat.T)
                fnd = true
              end
            end
          end
        end
        if not fnd then print("(none found)") end
      else CM_List(args,shared,"shared storage") end
		else printe("Shared Storage not available.") end
		return
	end
    if args=="show" or cmd=="cmw" then
        CM_window:SetVisible( true )
        return
    end
	if cmd=="cmp" then
    if args=="?" then
      Vinny.Common.Help(help,cmd)
      return
    end
		if args=='C' then
			printh("Crafting professions:")
			for p,n in pairs(Cat) do
				print('  '..p..' = '..n)
      end
      return
    elseif args=='' then
      for n,t in pairs(CM_Settings.prof) do
        if next(t) then CM_Prof(n) end
      end
    else CM_Prof(args) end
		return
	end
  local tier = args:match(npat)
  if tier then
    tier = tonumber(tier)
    if tier>0 and tier<=#Tier then
      print(Tier[tier].." tier nodes are found in: "..Nodes[tier])
    else printe("Invalid tier.") end
    return
  end
  local cat,tier = args:match(tpat)
  if cat then
    tier = tonumber(tier)
    if tier>0 and tier<=#Tier and Cat[cat] then
      print(Cat[cat].." mats, tier "..tier)
      for id,item in pairs(Mats) do
        if item.T==tier and item.C:find(cat,1,true) then
          print(string.format(xlink,id,item.N))
        end
      end
    else printe("Invalid category or tier.") end
    return
  end
  local id,name = args:match(xpat)
	if not id then id,name = Vinny.Common.EII_ID(args) end
  if id then
    local item = Mats[id]
    if not item then
      printe(name.." is not recognized, id= "..id)
      return
    end
    local nr = name:match("^(%d+) ")
    nr = nr and tonumber(nr) or 1
    CM_Uses(id,item)
    return
  end
  if args=='t' then
    print("Crafting tiers:")
    for i,n in pairs(Tier) do
      print('  '..i..' = '..n)
    end
    return
  end
	if args~='' then printe("Unrecognized argument, "..args) return end
  cVoc(true)
end

Turbine.Shell.AddCommand( "cm;cmi;cmv;cms;cmp;cmw;cm?",CM_Command )

Plugins.CraftMats.Unload = function(sender,args)
	pname = player:GetName()
	if pname:sub(1,1)=="~" then return end -- session play?
	CM_Settings.player[pname] = player:GetLevel()
    Turbine.PluginData.Save(Turbine.DataScope.Server,"CraftMats_Settings",CM_Settings)
    print(CMv..", settings saved.")
end

-- Options panel
import "Vinny.Common.Options"
OP = Vinny.Common.Options_Init(print,CM_Settings,CM_window,"CraftMats_Settings") 

local IR = Vinny.Common.Options_Box(OP,30," include rares in lists")
if CM_Settings.rare then IR:SetChecked(true) end
IR.CheckedChanged = function( sender, args )
	CM_Settings.rare = sender:IsChecked()
	print((CM_Settings.rare and "En" or "Dis").."abled include rares.")
end

-- Help text
help = {
	pre = "CM",
  help = "Enter '/cm ?' for arguments, '/cm?' for commands",
	['?'] = "Unknown, enter '/cm help', '/cm?', or '/cm ?'",
	arg = {
		[" "] = "Record current crafting skills.",
    t = "List crafting tiers.",
		["n<#>"] = "list zone where the tier <#> materials spawn.",
		["<c>0"] = "List all craft mats usable by profession <c>.",
		["<c><#>"] = "List craft mats usable by prof<c>, tier<#>.",
		["<link>"] = "List tier, storage locations and craft uses.",
	},
	cmd = {
    cmp = {
      [" "] = "List crafting prof. and char's skills in it",
      C = "List profession codes and names",
      ["<c>"] = "List training in prof <c>",
    },
    cmi = {
      [" "] = "Record crafting materials in inventory",
      ["<c><#>"] = "List invertory mats for prof and tier",
    },
    cmv = {
      [" "] = "Record crafting materials in vault",
      ["<c><#>"] = "List vault mats for prof and tier",
    },
    cms = {
      [" "] = "Record crafting materials in shared storage",
      ["<c><#>"] = "List shared storage mats for prof and tier",
      xp = "List shared storage mats for craft XP",
    },
    cmw = "Open Craft Mats window.",
  },
}
